-- #############################################################################
-- #
-- # VSpeak ECU data display
-- #
-- # Fuel gauge, turbine status, RPM, pump voltage and EGT display
-- # Designed for VSpeak Projet Hornet III telemetry sensor
-- #
-- # Based on original code by Bernd Woköck
-- #
-- # Copyright (c) 2017, Jim Brown
-- #
-- # Released under MIT-license
-- #
-- # 1.00 - Initial release
-- # 1.01 - Added fuel announcement, added configurable test control
-- #
-- #############################################################################

-- Config
local sensorId = 0
local warningPercent = 15
local tankSize = 3000
local resetSW
local announceSW
local testSW
local testControl
local testMode = false

-- module locals
local sensorsAvailable = {}
local initAnimation = 0
local fuelInit = 0
local tankDiff = 0
local reset = false
local announce = false
local testModeIndex
local bingoFile = ""
local bingoPlayed = false

-- sensor values
local fuel = 0
local pump = 0
local egt = 0
local rpm = 0
local status = 0

local state = {
	[-30] = "Dev.Delay",
	[-10] = "Emergency",
	[-1]  = "Power Off",
	[0]   = "Trim Off",
	[1]   = "Ready",
	[10]  = "Cool Down",
	[20]  = "Slow Down",
	[-20] = "Slow Down",
	[30]  = "Standby",
	[31]  = "Prop Ignit",
	[32]  = "Prop Heat",
	[33]  = "Pump Start",
	[34]  = "Burner On",
	[35]  = "Fuel Ignit",
	[36]  = "Fuel Heat",
	[37]  = "Ramp Delay",
	[38]  = "Ramp Up",
	[40]  = "Steady",
	[41]  = "Cal Idle",
	[42]  = "Calibrate",
	[43]  = "Wait Acc",
	[44]  = "Go Idle",
	[50]  = "Auto",
	[51]  = "Auto-HC"
}

local function DrawFuelGauge(percentage,ox,oy)
  -- triangle
  lcd.drawLine(6+ox,5+oy,17+ox,5+oy)
  lcd.drawLine(17+ox,5+oy,17+ox,63+oy)
  lcd.drawLine(17+ox,63+oy,14+ox,63+oy)
  lcd.drawLine(14+ox,63+oy,6+ox,5+oy)

  -- gas station symbol
  lcd.drawRectangle(51+ox,31+oy,5,9)
  lcd.drawLine(52+ox,34+oy,54+ox,34+oy)
  lcd.drawLine(50+ox,39+oy,56+ox,39+oy)
  lcd.drawLine(57+ox,31+oy,59+ox,33+oy)
  lcd.drawLine(59+ox,33+oy,59+ox,37+oy)
  lcd.drawPoint(58+ox,38+oy)
  lcd.drawLine(57+ox,38+oy,57+ox,35+oy)
  lcd.drawPoint(56+ox,35+oy)
  lcd.drawText(51+ox,2+oy,"F",FONT_MINI)
  lcd.drawText(51+ox,54+oy,"E",FONT_MINI)

  -- fuel bar
  lcd.drawRectangle(21+ox,53+oy,25,11)  -- lowest bar segment
  lcd.drawRectangle(21+ox,41+oy,25,11)
  lcd.drawRectangle(21+ox,29+oy,25,11)
  lcd.drawRectangle(21+ox,17+oy,25,11)
  lcd.drawRectangle(21+ox,5+oy,25,11)   -- uppermost bar segment

  -- calc bar chart values
  local nSolidBar = math.floor(percentage / 20)
  local nFracBar = (percentage - nSolidBar * 20) / 20  -- 0.0 ... 1.0 for fractional bar
  local i
  -- solid bars
  for i = 0, nSolidBar - 1, 1 do
    lcd.drawFilledRectangle(21+ox,53-i*12+oy,25,11)
  end
  --  fractional bar
  local y = math.floor(53 - nSolidBar * 12 + (1 - nFracBar) * 11 + 0.5)
  lcd.drawFilledRectangle(21+ox,y+oy,25,11 * nFracBar)

end

local function DrawStatus(status,rpm,ox,oy)
	local text = state[status]
	if(text == nil or text == '') then text = "???" end
	lcd.drawText(0+ox+76-lcd.getTextWidth(FONT_MINI,text),2+oy,text,FONT_MINI)
	local rpmText = string.format("%d",rpm)
	lcd.drawText(0+ox+76-lcd.getTextWidth(FONT_BOLD,rpmText),17+oy,rpmText,FONT_BOLD)
end

local function DrawPumpEGT(pump,egt,ox,oy)
  lcd.drawText(0+ox,1+oy,"Pump",FONT_MINI)
  lcd.drawText(0+ox+76-lcd.getTextWidth(FONT_MINI,"EGT"),1+oy,"EGT",FONT_MINI)
  lcd.drawText(0+ox,13+oy,string.format("%.2f%s",pump,"V"),FONT_BOLD)
	local egtText = string.format("%d%s",egt,"C")
  lcd.drawText(0+ox+76-lcd.getTextWidth(FONT_BOLD,egtText),13+oy,egtText,FONT_BOLD)
end

local function DrawFuelLow(fuelPercentage,ox,oy)

  if(system.getTime() % 2 == 0) then -- blink every second
    -- triangle
	lcd.drawLine(6+ox,47+oy,32+ox,3+oy)
	lcd.drawLine(32+ox,3+oy,60+ox,47+oy)
	lcd.drawLine(60+ox,47+oy,6+ox,47+oy)

	lcd.drawLine(7+ox,47+oy,33+ox,3+oy)
	lcd.drawLine(33+ox,3+oy,61+ox,47+oy)
	lcd.drawLine(61+ox,46+oy,6+ox,46+oy)

	lcd.drawLine(8+ox,47+oy,34+ox,3+oy)
	lcd.drawLine(34+ox,3+oy,62+ox,47+oy)
	lcd.drawLine(62+ox,46+oy,8+ox,46+oy)
	lcd.drawText(31+ox,10+oy,"!",FONT_BIG)
  end

  -- percentage and warning
  local percentX = 20
  if(fuelPercentage < 10) then percentX = 23 end
  lcd.drawText(percentX+ox,28+oy,string.format("%d%s",fuelPercentage,"%"),FONT_BOLD)
  lcd.drawText(1+ox,49+oy,"Fuel Low",FONT_BOLD)

	if(not bingoPlayed and bingoFile ~= "") then
		system.playFile(bingoFile,AUDIO_IMMEDIATE)
		bingoPlayed = true
	end

end

-- Print the telemetry values
local function OnPrint(width,height)

	if(testMode) then
		if(fuel == 0 and initAnimation == 0) then
			if(testSW and system.getInputsVal(testSW) > 0) then
				fuel = 5000
				tankSize = 4000
			end
		end
		if(testControl and initAnimation == 2) then
			local testValue = system.getInputsVal(testControl)
			fuel = 5000 - (5000 * ((testValue + 1) / 2))
			pump = testValue + 1
			status = math.floor(testValue + 1)
			rpm = (testValue + 1) * 60000
			egt = (testValue + 1) * 350
		end
	end

	-- set tank difference and start animation
	if(fuel > 0 and initAnimation == 0) then
		initAnimation = 1
		tankDiff = fuel - tankSize
		if(tankDiff < 0) then tankDiff = 0 end
	end

  -- animation demo at startup - fill tank up
  if(initAnimation == 1) then
    fuelInit = fuelInit + tankSize / 10
		fuel = fuelInit
    if(fuelInit >= tankSize) then
  	  initAnimation = 2
    end
  end
	
	--reset fuel percentage
	if(resetSW and not reset and system.getInputsVal(resetSW) > 0) then
		tankDiff = fuel - tankSize
		if(tankDiff < 0) then tankDiff = 0 end
		reset = true
	end

	fuel = fuel - tankDiff
	
  local fuelPercentage = fuel / tankSize * 100
  if(fuelPercentage > 99) then fuelPercentage = 99 end
  if(fuelPercentage < 0) then fuelPercentage = 0 end
  if(fuelPercentage > warningPercent or initAnimation < 2) then
    DrawFuelGauge(fuelPercentage,1,0)
  else
    DrawFuelLow(fuelPercentage,1,0)
  end
	--announce fuel percentage
	if(announceSW and not announcing and system.getInputsVal(announceSW) > 0) then
		system.playNumber(fuelPercentage, 0, "%")
		announcing = true
	else
		if(announcing and system.getInputsVal(announceSW) < 0) then
			announcing = false
		end
	end

  -- field separator lines
  lcd.drawLine(68,2,68,66)
  lcd.drawLine(68,36,148,36)

  -- turbine
  DrawStatus(status,rpm,72,0)

  -- pump/egt
  DrawPumpEGT(pump,egt,72,37)

end

--------------------------------------------------------------------
-- Form functions
--------------------------------------------------------------------
local function sensorChanged(value)

  if(value and value >= 0) then
    sensorId = sensorsAvailable[value].id
  else
    sensorId = 0
  end
  system.pSave("SensorId",sensorId)

end

local function warningChanged(value)
  warningPercent = value
  system.pSave("WarnigPercent",warningPercent)
end

local function tankSizeChanged(value)
  tankSize = value
  system.pSave("TankSize",tankSize)
end

local function resetSWChanged(value)
	resetSW = value
	system.pSave("ResetSwitch",resetSW)
end

local function announceSWChanged(value)
	announceSW = value
	system.pSave("AnnounceSwitch",resetSW)
end

local function fileChanged(value)
	bingoFile=value
	system.pSave("BingoFile",value)
end

local function testModeClicked(value)
  testMode = not value
  form.setValue(testModeIndex,testMode)
end

local function testSWChanged(value)
	testSW = value
	system.pSave("TestSwitch",testSW)
end

local function testControlChanged(value)
	testControl = value
	system.pSave("TestControl",testControl)
end

-- Init Form
local function initForm(formID)

  sensorsAvailable = {}
  local available = system.getSensors();
  local list = {}
  local curIndex = -1
  local descr = ""
  for index,sensor in ipairs(available) do
    if(sensor.param == 0) then
      list[#list + 1] = sensor.label
      sensorsAvailable[#sensorsAvailable + 1] = sensor
      if(sensor.id == sensorId ) then
        curIndex = #sensorsAvailable
      end
    end
  end
  -- sensor select
  form.addRow(2)
  form.addLabel({label = "Select sensor"})
  form.addSelectbox (list,curIndex,true,sensorChanged)
  -- tank volume
  form.addRow(2)
  form.addLabel({label = "Tank size (ml)"})
  form.addIntbox(tankSize,100,5000,3000,0,50,tankSizeChanged)
	-- fuel reset switch
	form.addRow(2)
	form.addLabel({label="Fuel Reset Switch"})
	form.addInputbox(resetSW,false,resetSWChanged)
	-- fuel announce switch
	form.addRow(2)
	form.addLabel({label="Fuel Announce Switch",width = 220})
	form.addInputbox(announceSW,false,announceSWChanged)
  -- fuel warning threshold
  form.addRow(2)
  form.addLabel({label = "Fuel warning (%)"})
  form.addIntbox(warningPercent,0,100,15,0,5,warningChanged)
	-- fuel warning audio file
	form.addRow(2)
	form.addLabel({label="Select warning file"})
	form.addAudioFilebox(bingoFile or "",fileChanged)
  -- test mode
  form.addRow(2)
  form.addLabel({label = "Test mode",width = 275})
  testModeIndex = form.addCheckbox(testMode,testModeClicked)
	-- test mode start switch
	form.addRow(2)
	form.addLabel({label="Test Mode Start Switch",width = 220})
	form.addInputbox(testSW,false,testSWChanged)
	-- test mode control
	form.addRow(2)
	form.addLabel({label="Test Mode Control"})
	form.addInputbox(testControl,true,testControlChanged)

end

--------------------------------------------------------------------
-- Init
--------------------------------------------------------------------
local function init()

  -- sensor id
  sensorId = system.pLoad("SensorId",0)

  -- other settings
  tankSize      	= system.pLoad("TankSize",4000)
  warningPercent	= system.pLoad("WarningPercent",15)
	resetSW					= system.pLoad("ResetSwitch")
	announceSW			= system.pLoad("AnnounceSwitch")
	testSW					= system.pLoad("TestSwitch")
	testControl			= system.pLoad("TestControl")
	bingoFile			 	= system.pLoad("BingoFile","")

  -- init values for startup animation
  initAnimation = 0
  fuel   = 0   -- empty
  status = -1  -- power off
  rpm    = 0

  system.registerTelemetry(2,"VSpeak ECU Data",2,OnPrint)
  system.registerForm(1,MENU_TELEMETRY,"VSpeak ECU Data",initForm,nil,printForm)
end

--------------------------------------------------------------------
-- Loop
--------------------------------------------------------------------
local function loop()
	
	if(not testMode) then
		local sensor
		-- Fuel
		sensor = system.getSensorByID(sensorId,6)
		if(sensor and sensor.valid) then fuel = sensor.value else fuel = 0 initAnimation = 0 end
		-- Pump Voltage
		sensor = system.getSensorByID(sensorId,4)
		if(sensor and sensor.valid) then pump = sensor.value else pump = 0 end
		-- EGT
		sensor = system.getSensorByID(sensorId,1)
		if(sensor and sensor.valid) then egt = sensor.value else egt = 0 end
		-- RPM
		sensor = system.getSensorByID(sensorId,2)
		if(sensor and sensor.valid) then rpm = sensor.value else rpm = 0 end
		-- ECU status
		sensor = system.getSensorByID(sensorId,3)
		if(sensor and sensor.valid) then status = sensor.value else status = -1 end
	end
--	local mem = collectgarbage('count')
--	print("mem ", mem)

end

--------------------------------------------------------------------------------
return {init=init,loop=loop,author="Jim Brown",version="1.00",name="VSpeak ECU Data"}
